#!/bin/bash
#
# build_pandora.sh

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_directory> <e17|e17|c2|c7>[:py3] <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=pandora
origpkgver=v03_11_01
pkgver=${origpkgver}e
ssibuildshims_version=v1_04_13
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pndver=`echo ${origpkgver} | sed -e 's/_/-/g'`
pkgtarfile=${package}-${pndver}.tar
PandoraPFA_ver=`echo ${origpkgver} | sed -e 's/_/-/g'`
PandoraSDK_ver=v03-03-02
PandoraMonitoring_ver=v03-04-01

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel";;
* )
   echo "ERROR: please specify debug or prof"
   usage
   exit 1
;;
esac

if [[ "${basequal}" == e[79] ]] || [[ "${basequal}" == e[79]:nu ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e1[045] ]] || [[ "${basequal}" == e1[045]:nu ]]
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++14"
elif [[ "${basequal}" == e1[79] ]] 
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++17 -Wno-implicit-fallthrough"
elif [[ "${basequal}" == e1[79]:py3 ]] 
then
  cc=gcc
  cxx=g++
  cxxflg="-std=c++17 -Wno-implicit-fallthrough"
elif [[ "${basequal}" == c[27] ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17 -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
elif [[ "${basequal}" == c[27]:py3 ]]
then
  cc=clang
  cxx=clang++
  cxxflg="-std=c++17 -D_LIBCPP_ENABLE_CXX17_REMOVED_FEATURES=1"
else
   echo "ERROR: unrecognized qualifier ${basequal}"
   usage
   exit 1
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup ${package} ${pkgver} failed."

source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup cmake"

mkdir -p ${PANDORA_FQ_DIR} || ssi_die "ERROR: failed to create ${PANDORA_FQ_DIR}"
mkdir -p ${blddir} || ssi_die "ERROR: failed to create ${blddir}"

if [ -z ${ROOTSYS} ]
then
   echo "ERROR: failed to setup root"
   exit 1
else
   echo "ROOTSYS: ${ROOTSYS}"
   echo
fi

mkdir -p ${PANDORA_FQ_DIR}/${package}-${pndver} || ssi_die "ERROR: failed to create ${PANDORA_FQ_DIR}/${package}-${pndver}"
cd ${PANDORA_FQ_DIR}/${package}-${pndver}
for rep in PandoraPFA PandoraSDK PandoraMonitoring
do
  tar xf ${tardir}/${rep}-*.tar  || ssi_die "failed to unwind ${tardir}/${rep}-*.tar"
done

cd ${blddir}
# we have to pass the compiler info to the sub-builds
export CC=${cc} 
export CXX=${cxx}
export FC=gfortran
for rep in PandoraSDK PandoraMonitoring
do
  mkdir ${blddir}/${rep} || ssi_die "Failed to create directory ${blddir}/${rep}"
  cd ${blddir}/${rep}
  rep_ver=${rep}_ver
  set -x
  cmake ${extra_command}  \
       -DCMAKE_MODULE_PATH=${PANDORA_FQ_DIR}/${package}-${pndver}/PandoraPFA-${PandoraPFA_ver}/cmakemodules \
       -DCMAKE_INSTALL_PREFIX=${PANDORA_FQ_DIR} \
       -DROOT_DIR=${ROOTSYS}/etc/cmake \
       -DCMAKE_CXX_FLAGS="${cxxflg}" \
       ${PANDORA_FQ_DIR}/${package}-${pndver}/${rep}-${!rep_ver} || ssi_die "cmake for ${rep} failed"
  make -j${ncore} install
  set +x
done

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: build failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup -B ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup failed."
echo "${package} is installed at ${PANDORA_FQ_DIR}"

# fix the cmake package Config files
# ${PANDORA_FQ_DIR}/*.cmake
# ${PANDORA_FQ_DIR}/lib/cmake/*.cmake

fixfile=/tmp/pandorafix$$$$.sh
rm -f ${fixfile}

echo "#!/bin/bash" > ${fixfile}
##echo " set -x " >>  ${fixfile}

cmakeConfig=`ls ${PANDORA_FQ_DIR}/*.cmake ${PANDORA_FQ_DIR}/lib/cmake/*.cmake`
for file in ${cmakeConfig}
do
  echo "mv ${file} ${file}.bak" >> ${fixfile}
  echo "cat ${file}.bak | sed 's%${PANDORA_FQ_DIR}%\$ENV{PANDORA_FQ_DIR}%g' | sed 's%${ROOTSYS}%\$ENV{ROOTSYS}%g' > ${file}" >> ${fixfile}
done

chmod +x ${fixfile}
${fixfile} || ssi_die "ERROR: problem running ${fixfile}."

rm -f ${fixfile}
rm -f ${PANDORA_FQ_DIR}/*.cmake.bak


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
